
c    Copyright 2003 Lars Ottemoller and Jens Havskov
c
c    This file is part of SEISNET.
c
c    SEISNET is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    Foobar is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with Foobar; if not, write to the Free Software
c    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USa

c
c This file contains SEISNET FORTRAN subroutines that are used for FTP server 
c type stations.
c

c
c List of subroutine in alphabetical order:
c
c   ftp_cnordic - convert file list output to Nordic file format
c   update_ftp - update s-file after waveform transfer
c

c
c Changes:
c
c  28 Nov 2001 lo: added documentation and clean-up
c

c

      subroutine ftp_cnordic(station_name,base,lastevent,
     &   pfile,stopbefore,event_dir,wave_req_dir,det_wav_option)

c
c convert a list of waveform files to SEISAN Nordic format
c
c Lars Ottemoeller, Jan 1997.
c

      implicit none

      include 'seidim.inc'
      integer  sei clen      ! character length

c -- station name
      character*5 base
c filename pattern
      character*40 filename_pattern
c name of parameter file
      character*80 pfile
c -- time
      integer hour, minute,sec
      real seconds,rse
c -- date
      integer year,month,day,jday
      integer ye,mo,da,jd,ho,mi,se
      double precision msec
c -- date and time of last event
      integer last_hour, last_minute,last_sec
      integer last_year,last_month,last_day
c -- linetype in Nordic format
c    distance indicator
      character*1 linetype, dist
c -- reporting agency
      character*3 agency
c -- line of QUAKE and nordic file resp.
      character*80 iline,line,nordicline,phaseline,idline
c -- top-directory
       character*60 top_directory
c -- index
       integer ind,inds
c -- directory separator character
      character*1 dchar
c -- text strings
       character*80 chr_text_out
c -- number of events
       integer counter
c -- instrument type in Nordic format
       character*1 instout
c -- idnumber
       character*14 idnumber,idtime
c -- counter
       integer i
c -- waveform file line
       character*80 waveline
c time (day,hour,min)
       character*6 time
c length of line
       integer l
c coda length - duration
       integer coda
c  name of S-file
       character*80 sfile
c station name
       character*5 station_name
c number of events
       integer nevent
c file exists and some flag
       logical exist,flag
c wav text
       character*42 wav_text
c definition of date and time in filename
       integer def(12)
c time to stop before now
       integer stopbefore
c option for waveform transfer
       integer det_wav_option
c directories for network event detection and waveform transfer
       character*(*) event_dir,wave_req_dir

       character*40 lastevent ! last event in data base
       character*40 currentevent ! current event in data base
       character*40 now_time,check_time
       integer      stat     ! flag to compare date of events
       integer  length       ! length of sfile nam,e
       character*14 lastdetection ! most recent detection


      nevent=0

c
c get directory structure
c

      call topdir(top_directory)

      ind=seiclen(top_directory)
      inds=seiclen(station_name)
      if (inds.eq.-1) inds=4

      call dir_char(dchar)         ! dirctory delimiter character

      call find_file_def(pfile,station_name,def,filename_pattern)

      if (filename_pattern.eq.' ') then
        write(*,*) 'bad definition of filename pattern'
        return
      endif

c
c check if def ok
c
      flag = .true.
      do i=1,12
c        write(*,*) def(i)
        if (def(i).eq.0) flag=.false.
      enddo
   
      if (.not.flag) then
        write(*,*) 'bad definition of filename'
        return
      endif

c
c open files
c
      call file_ready(station_name(1:inds)//'.INP')
      open(50,file=station_name(1:inds)//'.INP',status='old')
      ind=index(chr_text_out,' ')-1

c
c set default parameters
c
      agency=station_name(1:3)
      linetype= '1'

      counter=0


c
c get date and time of last event
c
      read(lastevent(1:4),'(i4)') last_year
      read(lastevent(5:6),'(i2)') last_month
      read(lastevent(7:8),'(i2)') last_day
      read(lastevent(9:10),'(i2)') last_hour
      read(lastevent(11:12),'(i2)') last_minute
      read(lastevent(13:14),'(i2)') last_sec

      lastdetection=lastevent
     

  10  continue
c
c read original line
c
      do i=1,80
        line(i:i)=' '
      enddo
      read(50,'(a80)',end=900) iline

      line(1:seiclen(iline)) = iline(1:seiclen(iline))
c
c check if line fits pattern
c
      do i=1,seiclen(filename_pattern)
        if (filename_pattern(i:i).ne.'?') then
          if (line(i:i).eq.filename_pattern(i:i)) then

          else
c            write(*,*) ' mismatch ',line,i
            goto 10
          endif
        endif
      enddo

      counter=counter+1

      wav_text=line(1:seiclen(line))
      call write_waveline(waveline,'EVE',station_name,'FTP',
     &     wav_text(1:seiclen(wav_text)))

      read(line(def(1):def(2)),'(i4)') year
        
c
c  add century
c
      if (year.lt.100) then
        if (year.gt.50) then
           year = year + 1900
        else
           year = year + 2000
        endif
      endif

c
c in case of VME Seislog, time can be 8 years behind
c
      if (year.lt.2000) year=year+8
c month and day, or julian day
      if (def(3).ne.-1.and.def(4).ne.-1) then
        read(line(def(3):def(4)),'(i2)') month
        read(line(def(5):def(6)),'(i2)') day
      else
        read(line(def(5):def(6)),'(i3)') jday
        call dte(jday,day,month,year)
      endif

      read(line(def(7):def(8)),'(i2)') hour
      read(line(def(9):def(10)),'(i2)') minute
      read(line(def(11):def(12)),'(i2)') sec

      write(currentevent,'(i4,5(i2))') year,month,day,hour,
     *minute,sec
      do i=1,14
         if (currentevent(i:i).eq.' ') then
           currentevent(i:i)= '0'
         endif
      enddo

c
c decide to convert or not
c if date of current is newer (greater) than date of last event
c then convert it, otherwise, bypass
c
      call compare(currentevent,lastevent,stat)
      if (stat.le.0) goto 25
c
c set most recent event detection
c
      lastdetection=currentevent

c
c check if file wanted, only if older then nowtime - stopbefore
c
      ye=year
      mo=month
      jd=jday
      da=day
      ho=hour
      mi=minute
      se=sec
      rse=se
      call timsec(ye,mo,da,ho,mi,rse,msec)
      msec = msec + stopbefore

      call sectim(msec,ye,jd,mo,da,ho,mi,rse)
      se=int(rse)
      write(check_time,'(i4,5(i2))') ye,mo,da,ho,
     *mi,se


      do i=1,14
         if (check_time(i:i).eq.' ') then
           check_time(i:i)= '0'
         endif
      enddo

      call gmttim(now_time)

      call compare(now_time,check_time,stat)
      if (stat.le.0) goto 25


  15  continue

      instout='S'
      dist= 'R'

c
c write nordic line
c
      nordicline= ' '

      write(nordicline(2:5),'(i4)') year
      write(nordicline(7:8),'(i2)') month
      write(nordicline(9:10),'(i2)') day
      write(nordicline(12:13),'(i2)') hour
      write(nordicline(14:15),'(i2)') minute
      write(nordicline(17:18),'(i2)') sec
      write(nordicline(22:22),'(a1)') dist
c      write(nordicline(45:45),'(a1)') '*'
      write(nordicline(46:48),'(a3)') agency
      write(nordicline(51:51),'(a1)') '1'
      write(nordicline(80:80),'(a1)') linetype


c
c  write idline
c
      call write_idline(idline,year,month,day,hour,minute,sec)

      phaseline=' '
      write(phaseline(2:6),'(a5)') station_name
      write(phaseline(7:7),'(a1)') instout
      write(phaseline(8:8),'(a1)') 'Z'
      write(phaseline(10:11),'(a2)') 'EP'
      write(phaseline(16:16),'(a1)') 'A'
      write(phaseline(19:20),'(i2)') hour
      write(phaseline(21:22),'(i2)') minute
      write(phaseline(24:28),'(f5.2)') seconds
      write(phaseline(30:33),'(i4)') coda

c
c open output file
c

c      if (year.gt.1900) year=year-1900

      call sfilname(year,month,day,hour,
     &      minute,sec,base,dist,sfile,length )

c
c  skip if file exists
c
      inquire(file=sfile,exist=exist)

      if (exist) goto 20

      call file_ready(sfile)

      open(60,file=sfile,status='new')
c
c output
c
      write(*,*) nordicline
      write(60,'(a80)') nordicline
      write(60,'(a80)') idline
      write(60,'(a80)') waveline
      write(60,'(2a40)') ' STAT SP IPHASW D HRMM SECON CODA AMPLIT',
     &   ' PERI AZIMU VELO SNR AR TRES W  DIS CAZ7'
c      write(60,'(a80)') phaseline
      write(60,*)

      close(60)

      call copy_det_file(sfile,event_dir,wave_req_dir,det_wav_option)

      nevent=nevent + 1

  20  CONTINUE
      goto 10

  25  continue
c     write(*,*) 'BYPASS ...'
      goto 10

 900  continue

      close(50)
      close(60)

      write(*,*) '... in total ',nevent,' detections ...'
      write(*,*) '... last detection ',lastdetection,' ...'
 
      return
      end


      subroutine update_ftp(s_file,wave_ftp,wave_name,autopick)
c
c update s-file after waveform transfer
c

      implicit none

      include 'libsei.inc'                ! Open file definitions
      include 'seidim.inc'

      character*255 S_file              ! name of S-file
      character*42  wave_name,wave_ftp  ! name of waveform file
      character*8 autopick              ! name of autopick routine

      logical      there                ! test if file exists

c-- event data
       character*80     data(max_data)
c-- exp indicator
       character*1      exp
c-- event type
       character*1      type
c-- number of recors for event
       integer          nrecord
c-- number of header lines
       integer          nhead
c number of phases
       integer nphase
c read unit
       integer read01
c number of stations
       integer nstat
c-- id line number
       integer id
c counter
      integer x
c line with 'REQUEST'
      integer requ_line
c waveform line
      character*80 waveline
c autopick line
      character*80 autopick_line
c index
      integer ind
c flag if autopick line
      logical flag
      integer seiclen



c
c   check if file exists
c
      inquire (file=S_file,exist=there)
      if (.NOT.THERE) then
         write(*,*) S_file,' does not exist !'
         return
      endif


c
c  open S-file
c
      call file_ready(S_file)
      open(unit=8,file=S_file,status='old')

c
c read event file
c
      read01=8
      call indata(read01,nstat,nphase,nhead,nrecord,type,exp,data,id)

      close(8)

c
c look for "REQUEST"
c
      do x=1,nhead
        if (data(x)(66:72).eq.'REQUEST'.or.
     &      data(x)(76:78).eq.'REQ') then
          if (data(x)(23:23+seiclen(wave_ftp)).eq.
     &             wave_ftp(1:seiclen(wave_ftp))) then
            requ_line=x
          endif
        endif
      enddo

c
c  make autopick_line
c
cNET WAV EVE TRO  FTP 9809_05_0000_58T.TRO_03_1                  AUTOPICK

c lot 9/9/2002
      if (wave_name.eq.'never') then
        flag=.false.
        data(requ_line)(76:78) = 'NEV' 
      else
        call write_autopick_line(data(requ_line),autopick_line,
     &    wave_name,autopick,flag)

c
c update line
c
c        data(requ_line)(66:76) = 'TRANSFERRED'
        data(requ_line)(76:78) = 'TRA'   ! changed April 2001, lo

c
c blank new line
c
        do x=1,80
           waveline(x:x) = ' '
        enddo
c
c write info to new line
c
        write(waveline(2:43),'(a42)') wave_name
        write(waveline(80:80),'(a1)') '6'
      endif
c
c update according to status of file transfer
c

        call file_ready(S_file)
        open(8,file=S_file,status='unknown')

        do x=1,nhead-1
           write(8,'(a80)') data(x)
        enddo

        if (flag) write(8,'(a80)') autopick_line

        write(8,'(a80)') waveline

        do x=nhead,nrecord
          write(8,'(a80)') data(x)
        enddo
c
c close updated S-file and end process
c

        close(8)


      return
      end

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc




